/*
This sample program shows how to use the ShortCuts class to give
users the ability to create custom shortcuts on the main dialog
window and/or a toolbar.

For best results, each menu item and sub-menu should have a unique
ID and a bitmap.  Shortcuts are saved in a file named ShortCuts.Ar.
*/


#INCLUDE "dcdialog.CH"
#include "dcbitmap.ch"
#INCLUDE "dccursor.CH"
#INCLUDE "appevent.CH"

FUNCTION Main()

LOCAL GetList[0], oShortCuts, GetOptions

DC_BitmapTransparentColor( { 192, 192, 192 } )

BuildMenu( GetList, @oShortCuts )

oShortCuts := ShortCuts():new(GetList):BuildShortcuts()

DCGETOPTIONS RESIZE
DCREAD GUI FIT TITLE 'Edit and Save Desktop Shortcuts' SETAPPWINDOW ;
       OPTIONS GetOptions HANDLER myHandler

RETURN nil

* ----------

PROC appsys ; RETURN

* ----------

STATIC FUNCTION myHandler( nEvent, mp1, mp2, oXbp )

/*
LOCAL lStatus

IF oXbp#nil .AND. oXbp:isDerivedFrom('DC_XbpMenu') .AND. nEvent == 1048610
  wtf oXbp:items[mp1]
  IF AppKeyState(xbeK_SHIFT) == 1
    DCMSGBOX 'Add this menu item to custom buttons?' YESNO TO lStatus
    RETURN DCGUI_IGNORE
  ENDIF
ENDIF

*/

RETURN DCGUI_NONE

* ----------

STATIC FUNCTION BuildMenu( GetList, oShortCuts )

LOCAL oMenuBar, oMenu1, oMenu2, oMenu3

DCMENUBAR oMenuBar OWNERDRAW

  DCSUBMENU oMenu1 PROMPT 'File' PARENT oMenuBar ID 'MENU_ITEM_FILE' ;
    MESSAGE 'File Sub Menu' BITMAP BITMAP_FOLDEROPEN_M

    DCMENUITEM 'Bug' PARENTID 'MENU_ITEM_FILE' ACTION {||MsgBox('I ate a bug')} ID 'MENU_ITEM_BUG' ;
       MESSAGE 'Eat a bug' BITMAP BITMAP_BUG_M
    DCMENUITEM 'E-mail' PARENTID 'MENU_ITEM_FILE' ACTION {||MsgBox('Email sent!')} ;
       ID 'MENU_ITEM_EMAIL' ;
       MESSAGE 'Send an Email' BITMAP BITMAP_MAIL_M
    DCMENUITEM SEPARATOR PARENTID 'MENU_ITEM_FILE'
    DCMENUITEM 'Quit' PARENTID 'MENU_ITEM_FILE' ACTION {||DC_ReadGuiEvent(DCGUI_EXIT_OK,GetList)} ;
       ID 'MENU_ITEM_QUIT' ;
       MESSAGE 'Quit this program' BITMAP BITMAP_EXIT_M

  DCSUBMENU oMenu2 PROMPT 'Design' PARENT oMenuBar ID 'MENU_ITEM_DESIGN' ;
    MESSAGE 'Design Sub Menu' BITMAP BITMAP_TEMPLATE_M

    DCMENUITEM 'Build Project' PARENTID 'MENU_ITEM_DESIGN' ACTION {||MsgBox('Build Project')} ID 'MENU_ITEM_BUILDPROJECT' ;
      MESSAGE 'Create a new Project' BITMAP BITMAP_PROJECT_M
    DCMENUITEM 'Print Project' PARENTID 'MENU_ITEM_DESIGN' ACTION {||MsgBox('Print Project')} ID 'MENU_ITEM_PRINTPROJECT' ;
      MESSAGE 'Print the built Project' BITMAP BITMAP_PRINT_M
    DCMENUITEM SEPARATOR PARENTID 'MENU_ITEM_DESIGN'
    DCMENUITEM 'Copy Project to ClipBoard' PARENTID 'MENU_ITEM_DESIGN' ACTION {||MsgBox('Copy Text')} ID 'MENU_ITEM_COPYPROJECT' ;
      MESSAGE 'Copy the entire project to the Clipboard' BITMAP BITMAP_COPYCLIP_M

  DCSUBMENU oMenu3 PROMPT 'Help' PARENT oMenuBar ID 'MENU_ITEM_HELP' ;
      BITMAP BITMAP_HELP_M MESSAGE 'Help Sub Menu'

    DCMENUITEM 'Find Help' PARENTID 'MENU_ITEM_HELP' ACTION {||MsgBox('Find Help')} ;
      ID 'MENU_ITEM_FINDHELP' MESSAGE 'Find the Help file' ;
      BITMAP BITMAP_FIND_M
    DCMENUITEM SEPARATOR PARENTID 'MENU_ITEM_HELP'
    DCMENUITEM 'About' PARENTID 'MENU_ITEM_HELP' ACTION {||MsgBox('About')} ID 'MENU_ITEM_ABOUT' ;
      MESSAGE 'About this ShortCuts program' BITMAP BITMAP_HEX_M

RETURN nil

* ----------

CLASS ShortCuts

EXPORTED:

VAR shortCutArray
VAR getList
VAR toolBarObject
VAR staticObject
VAR popMenuObject
VAR statusObject
VAR toolButtonSize
VAR desktopButtonSize
VAR deleteButtonObject
VAR configArray
VAR isSnapToGrid
VAR statusAlignment
VAR shortCutFilePath
VAR isAddEditIcon
VAR isEditing

METHOD Init, BuildShortCuts, BuildShortCut, DeleteButton, EditShortCuts, DropIcon, ;
       ShortCutConfig, SaveShortCuts, LoadShortCuts, FixMenuItems

ENDCLASS

* -----------

METHOD ShortCuts:init( GetList )

LOCAL cUserName := Space(100), nLen := 100

::configArray := Array(2)
::toolButtonSize := 35
::deskTopButtonSize := 60
::getList := GetList
::isSnapToGrid := .t.
::shortCutConfig()
::statusAlignment := DCGUI_ALIGN_LEFT
::shortCutFilePath := DC_Path(AppName(.t.))
::isAddEditIcon := .t.
::isEditing := .f.

RETURN self

* -----------

METHOD ShortCuts:LoadShortCuts()

::shortCutArray := DC_ARestore(::shortCutFilePath+'ShortCuts.Ar')
IF ::shortCutArray == nil
  ::shortCutArray := {}
ENDIF

RETURN self

* -----------

METHOD ShortCuts:SaveShortCuts()

DC_ASave(::shortCutArray,::shortCutFilePath+'Shortcuts.Ar')

RETURN self

* ----------

METHOD ShortCuts:EditShortCuts()

LOCAL aMenu[0], oBrowse, i, GetList[0], GetOptions, ;
      aMessage, cMessage, aBitmap, oBitmap, nPointer := 0, lStatus

::isEditing := .t.

FOR i := 1 TO Len(::getList)

  IF ::getList[i,nGETLIST_TYPE] $ { GETLIST_SUBMENU, GETLIST_MENUITEM } .AND. ;
      !Empty(::getList[i,cGETLIST_CAPTION]) .AND. !Empty(::getList[i,cGETLIST_ID])

    aMessage := ::getList[i,cGETLIST_MESSAGE]
    IF Empty(aMessage)
      cMessage := ''
    ELSE
      cMessage := aMessage[1]
    ENDIF
    aBitmap := ::getList[i,xGETLIST_OPTIONS2]
    IF !Empty(aBitmap)
      oBitmap := DC_GetBitmap(aBitmap[1])
    ELSE
      oBitmap := DC_GetBitmap(BITMAP_OBJECT_M)
    ENDIF

    AAdd( aMenu, { ::getList[i,cGETLIST_ID], ;
                   ::getList[i,nGETLIST_TYPE], ;
                   ::getList[i,cGETLIST_CAPTION], ;
                   ::getList[i,bGETLIST_ACTION], ;
                   cMessage, ;
                   oBitmap } )

  ENDIF

NEXT

@ 0,0 DCSAY 'Click on Icon and drag to Desktop or ToolBar' SAYSIZE 0 ;
      FONT '10.Arial Bold'

@ 1,0 DCBROWSE oBrowse DATA aMenu SIZE 20,20 FIT POINTER nPointer ;
      RESIZE DCGUI_RESIZE_RESIZEONLY

DCBROWSECOL ELEMENT 6 HEADER 'Icon' WIDTH 4 PARENT oBrowse TYPE XBPCOL_TYPE_BITMAP ;
     DRAG {|o|::staticObject:dropCursor := POINTER_DATADROP_COPY,{Eval(o:dataLink),aMenu,nPointer,nil}} ;
       DIALOG {|mp1,mp2,oXbp|DragDialog(mp1,mp2,oXbp)} ;

DCBROWSECOL DATA {||IIF(aMenu[oBrowse:arrayElement,2]==27,'Item','Menu')} HEADER 'Type' WIDTH 4 PARENT oBrowse
DCBROWSECOL ELEMENT 3 HEADER 'Caption' WIDTH 12 PARENT oBrowse
DCBROWSECOL ELEMENT 5 HEADER 'Message' WIDTH 15 PARENT oBrowse
DCBROWSECOL ELEMENT 1 HEADER 'ID' WIDTH 20 PARENT oBrowse

DCGETOPTIONS RESIZE
DCREAD GUI FIT TITLE 'Drag ShortCut to Desktop / ToolBar' OPTIONS GetOptions

::saveShortCuts()

::isEditing := .f.
DC_GetWhen(::getList)

RETURN nil

* -------------

METHOD ShortCuts:DropIcon( oStatic, aData, aPos )

LOCAL oBitmap, aMenu, nPointer, GetList[0], cId, oButton, lStatus := .t., ;
      lToolBar, nGridCols := ::deskTopButtonSize, nGridRows := ::deskTopButtonSize, ;
      nFound

oBitmap := aData[1]
aMenu := aData[2]
nPointer := aData[3]
oButton := aData[4]

lToolBar := oStatic:isDerivedFrom('DC_XbpToolBar')

IF Valtype(oButton) == 'O'
  cId := DC_GetCargo(oButton)
  nFound := 0
ELSE
  cId := aMenu[nPointer,1]
  nFound := AScan(::shortCutArray,{|a|a[1]==cID .AND. a[4]==lToolBar})
ENDIF

IF nFound > 0
  DCMSGBOX 'Shortcut already exists on ' + IIF(lToolBar,'ToolBar','DeskTop')
  RETURN .f.
ENDIF

IF Valtype(oButton) == 'O'
  ::deleteButtonObject := oButton
  ::deleteButton()
ENDIF

aPos[2] := oStatic:currentSize()[2] - aPos[2]
IF ::isSnapToGrid
  aPos[1] := Round( aPos[1] / nGridCols, 0 ) * nGridCols
  aPos[2] := Round( aPos[2] / nGridRows, 0 ) * nGridRows
ENDIF

::BuildShortCut( GetList, cId, lToolBar, { cId, aPos[2], aPos[1] } )
DCREAD GUI PARENT oStatic EXIT
DC_MergeGetLists( ::getList, GetList )
AAdd( ::shortCutArray, { cId, aPos[2], aPos[1], lToolBar } )
::saveShortCuts()

RETURN .t.

* -----------

METHOD ShortCuts:FixMenuItems()

LOCAL i, cCaption

FOR i := 1 TO Len(::getList)
  IF ::getList[i,nGETLIST_TYPE] $ { GETLIST_SUBMENU, GETLIST_MENUITEM } .AND. ;
      !Empty(::getList[i,cGETLIST_CAPTION]) .AND. Empty(::getList[i,cGETLIST_ID])
    cCaption := ::getList[i,cGETLIST_CAPTION]
    cCaption := Strtran(cCaption,' ','_')
    ::getList[i,cGETLIST_ID] := cCaption
  ENDIF
NEXT

RETURN nil

* -----------

METHOD ShortCuts:BuildShortCuts()

LOCAL i, lToolBar, cId, GetList := ::getList, lVertical

::fixMenuItems()
::loadShortCuts()

@ 0,0 DCSTATIC TYPE XBPSTATIC_TYPE_TEXT SIZE 800,600 PIXEL ;
        DROP {|o,a,b|::DropIcon(o,a,b)} OBJECT ::staticObject ;
        RESIZE DCGUI_RESIZE_RESIZEONLY

IF ::statusAlignment $ { DCGUI_ALIGN_LEFT, DCGUI_ALIGN_RIGHT }
  lVertical := .t.
  DCSTATUSBAR ::statusObject WIDTH ::toolButtonSize ;
    ALIGN ::statusAlignment TYPE XBPSTATIC_TYPE_TEXT
ELSE
  lVertical := .f.
  DCSTATUSBAR ::statusObject HEIGHT ::toolButtonSize ;
    ALIGN ::statusAlignment TYPE XBPSTATIC_TYPE_TEXT
ENDIF

@ 0,0 DCTOOLBAR ::toolBarObject ;
      SIZE IIF(lVertical,::toolButtonSize,1000), IIF(lVertical,1000,::toolButtonSize) ;
      BUTTONSIZE ::toolButtonSize, ::toolButtonSize ;
      TYPE XBPSTATIC_TYPE_TEXT PARENT ::statusObject PIXEL ;
      COLOR nil, GRA_CLR_WHITE ;
      DROP {|o,a,b|::DropIcon(o,a,b)}

DCSUBMENU ::popMenuObject
  DCMENUITEM 'Delete this ShortCut' PARENT ::popMenuObject ;
     ACTION {||::DeleteButton()}

IF ::isAddEditIcon

  DCADDBUTTONXP ;
     CAPTIONARRAY { { DC_GetBitmap(BITMAP_DESIGN_M),, 0,0,35,35,,,} } ;
     PARENT ::toolBarObject ;
     ACTION {|o|o:=Thread():new(),Sleep(5),o:start({||::EditShortCuts()})} ;
     ID 'MENU_ITEM_EDITSHORTCUTS' ;
     TOOLTIP 'Drag Menu Items to the Desktop and/or Toolbar' ;
     WHEN {||!::isEditing}

  DCADDBUTTON TYPE XBPSTATIC_TYPE_RAISEDBOX SIZE ::toolButtonSize,5 PARENT ::toolBarObject ;
     COLOR nil, GRA_CLR_PALEGRAY

ENDIF

IF Empty(::shortCutArray)
  RETURN self
ENDIF

FOR i := 1 TO Len(::shortCutArray)
  cId := ::shortCutArray[i,1]
  lToolBar := ::shortCutArray[i,4]
  ::BuildShortCut( GetList, cId, lToolBar, ::shortCutArray[i], ::shortCutArray[i] )
NEXT

RETURN self

* -----------

METHOD ShortCuts:BuildShortCut( GetList, cId, lToolBar, aShortCut )

LOCAL nFound, aBitmap, nSize, aCaptions, cCaption, nColorBG, ;
      bEval, oBitmap, bAction, cToolTip, bDrag, bDialog, lStatus := .t.

nFound := AScan(::getList,{|a|a[cGETLIST_ID]==cId})

IF nFound > 0 .AND. lStatus

  aBitmap := ::getList[nFound,xGETLIST_OPTIONS2]
  nSize := IIF(lToolBar,25,::deskTopButtonSize)
  IF !Empty(aBitmap)
    oBitmap := DC_GetBitmap(aBitmap[1])
    IF lToolBar
      aCaptions := { { oBitmap,, (::toolButtonSize-nSize)/2,(::toolButtonSize-nSize)/2,nSize,nSize,,,} }
    ELSE
      aCaptions := { { oBitmap,, 4,4,nSize-8,nSize-8,,,} }
    ENDIF
    cCaption := nil
    nColorBG := GRA_CLR_BACKGROUND
  ELSE
    aCaptions := nil
    cCaption := Strtran(::getList[nFound,cGETLIST_CAPTION],' ',Chr(13))
    nColorBG := GRA_CLR_CYAN
    aCaptions := { { cCaption,, 4,4,nSize-8,nSize-8,,,} }
  ENDIF

  bAction := ::getList[nFound,bGETLIST_ACTION]
  IF Empty(bAction) .AND. ::getList[nFound,nGETLIST_TYPE] == GETLIST_SUBMENU
    bAction := AnchorMenuBlock(cId,::getList)
  ENDIF
  cToolTip := ::getList[nFound,cGETLIST_MESSAGE]

  bEval := {|o|o:rbDown := {|a,b,o|::popMenuObject:popUp( o:setParent(), o:currentPos(), 2 , ;
                 XBPMENU_PU_DEFAULT + XBPMENU_PU_MOUSE_RBDOWN  ), ;
                 ::deleteButtonObject := o }}

  bDrag := {|o|::staticObject:dropCursor := POINTER_DATADROP_MOVE,{o,nil,nil,o}}
  bDialog := {|mp1,mp2,oXbp|DragDialog(mp1,mp2,oXbp)}

  IF lToolBar

     DCADDBUTTONXP ;
           ACTION bAction ;
           CAPTIONARRAY aCaptions ;
           CAPTION cCaption ;
           PARENT ::toolBarObject ;
           TOOLTIP cToolTip ;
           CARGO cId ;
           ID 'BUTTON_TOOLBAR_' + cId ;
           EVAL bEval ;
           CONFIG ::configArray[2] ;
           DRAG bDrag DIALOG bDialog

   ELSE

        @ aShortCut[2], aShortCut[3] DCPUSHBUTTONXP SIZE nSize,nSize PIXEL ;
           ACTION bAction ;
           CAPTIONARRAY aCaptions ;
           CAPTION cCaption ;
           PARENT ::staticObject ;
           TOOLTIP cToolTip ;
           CARGO cId ;
           EVAL bEval ;
           ID 'BUTTON_DESKTOP_' + cId ;
           CONFIG ::configArray[1] ;
           OUTLINE ;
           DRAG bDrag DIALOG bDialog ;
           RESIZE DCGUI_RESIZE_REPOSONLY_Y

   ENDIF
ENDIF

RETURN lStatus

* ------------

METHOD ShortCuts:DeleteButton()

LOCAL nFound, cId, lExact, lSaveShortCuts := .f., ;
      lToolBar := .f., cScanId, oButton, GetList := ::getList

oButton := ::deleteButtonObject
cId := DC_GetCargo(oButton)

IF oButton:setParent():isDerivedFrom("DC_XbpToolBar")
  lToolBar := .t.
  cScanId := 'BUTTON_TOOLBAR_' + cId
ELSE
  cScanId := 'BUTTON_DESKTOP_' + cId
ENDIF

oButton:destroy()

nFound := AScan(GetList,{|a|a[cGETLIST_ID]==cScanId})

IF nFound > 0
  ARemove( GetList, nFound )
ENDIF

lExact := Set(_SET_EXACT,.t.)
nFound := AScan(::shortCutArray,cID)
Set(_SET_EXACT,lExact)

IF nFound > 0
  ARemove( ::shortCutArray, nFound )
  ::saveShortCuts()
ENDIF

RETURN nil

* -------------

METHOD ShortCuts:ShortCutConfig( nWhich )

#define COLOR_RED        GraMakeRGBColor( { 255,   0,   0 } )
#define COLOR_DARKRED    GraMakeRGBColor( { 127,   0,   0 } )
#define COLOR_GREEN      GraMakeRGBColor( {   0, 255,   0 } )
#define COLOR_DARKGREEN  GraMakeRGBColor( {   0, 127,   0 } )
#define COLOR_BLUE       GraMakeRGBColor( {   0,   0, 255 } )
#define COLOR_DARKBLUE   GraMakeRGBColor( {   0,   0, 127 } )
#define COLOR_CYAN       GraMakeRGBColor( {   0, 255, 255 } )
#define COLOR_DARKCYAN   GraMakeRGBColor( {   0, 127, 127 } )
#define COLOR_YELLOW     GraMakeRGBColor( { 255, 255,   0 } )
#define COLOR_BROWN      GraMakeRGBColor( { 127, 127,   0 } )
#define COLOR_PINK       GraMakeRGBColor( { 255,   0, 255 } )
#define COLOR_DARKPINK   GraMakeRGBColor( { 127,   0, 127 } )
#define COLOR_WHITE      GraMakeRGBColor( { 255, 255, 255 } )
#define COLOR_BLACK      GraMakeRGBColor( {   0,   0,   0 } )
#define COLOR_PALEGRAY   GraMakeRGBColor( { 192, 192, 192 } )
#define COLOR_DARKGRAY   GraMakeRGBColor( { 127, 127, 127 } )
#define COLOR_ORANGE     GraMakeRGBColor( { 253, 120,  41 } )
#define COLOR_LTORANGE   GraMakeRGBColor( { 250, 205,   0 } )
#define COLOR_LTBLUE     GraMakeRGBColor( {  30, 187, 255 } )
#define COLOR_LTYELLOW   GraMakeRGBColor( { 128, 128,   0 } )
#define COLOR_DKYELLOW   GraMakeRGBColor( { 212, 212,   0 } )
#define COLOR_DKYELLOW2  GraMakeRGBColor( { 170, 170,   0 } )
#define COLOR_DKBROWN    GraMakeRGBColor( {  72,  72,   0 } )
#define COLOR_SLATEGRAY  GraMakeRGBColor( { 158, 182, 205 } )
#define COLOR_SILVER     GraMakeRGBColor( { 192, 192, 192 } )
#define COLOR_ICEGREEN   GraMakeRGBColor( { 222, 255, 189 } )

::configArray[1] := DC_XbpPushButtonXPConfig():new()
::configArray[1]:outline := .t.
::configArray[1]:radius := 10
::configArray[1]:gradientStep := 6
::configArray[1]:gradientLevel := 300
::configArray[1]:gradientReverse := .t.
::configArray[1]:bgColorMouse := COLOR_SILVER
::configArray[1]:bgColorClick := COLOR_ORANGE
::configArray[1]:isTransparent := .t.
::configArray[1]:shadowType := 8
::configArray[1]:font := '10.Arial'
::configArray[1]:bgColor := GRA_CLR_BACKGROUND

::configArray[2] := DC_XbpPushButtonXPConfig():new()
::configArray[2]:outline := .t.
::configArray[2]:radius := 0
::configArray[2]:gradientStep := 1
::configArray[2]:gradientLevel := 400
::configArray[2]:gradientReverse := .t.
::configArray[2]:bgColorMouse := COLOR_ICEGREEN
::configArray[2]:bgColorClick := COLOR_CYAN
::configArray[2]:isTransparent := .t.
::configArray[2]:font := '8.Arial'
::configArray[2]:bgColor := GRA_CLR_BACKGROUND

RETURN self

* ----------

STATIC FUNCTION AnchorMenuBlock( cId, GetList )

RETURN {|a,b,o|MenuAction( cId, GetList, o )}

* ----------

STATIC FUNCTION MenuAction( cId, GetList, o )

LOCAL oMenu, nFound

nFound := AScan(GetList,{|a|a[cGETLIST_ID]==cID})
oMenu := GetList[nFound,oGETLIST_OBJECT]

oMenu:popUp( o:setParent(), o:currentPos(), 2 , ;
                  XBPMENU_PU_DEFAULT + XBPMENU_PU_MOUSE_RBDOWN  )

RETURN nil

* -----------

STATIC FUNCTION DragDialog( mp1, mp2, oXbp )

LOCAL GetList[0], GetOptions, oDialog, aPos, cText, oBitmap, oStatic

IF oXbp:isDerivedFrom('XbpColumn')
  oBitmap := Eval(oXbp:dataLink)
ELSEIF oXbp:isDerivedFrom('DC_XbpPushButtonXP')
  oBitmap := oXbp:captionArray[1,1]
ENDIF

@ 2,0 DCSTATIC TYPE XBPSTATIC_TYPE_TEXT SIZE 60,60 PIXEL ;
      OBJECT oStatic

DCBITMAP oBitmap PARENT oStatic AUTOSCALE CENTER

DCGETOPTIONS NOTITLEBAR ORIGIN XBPDLG_ORIGIN_MOUSE ;
   FITPAD 0 COLOR { 240, 240, 255 } ;
   BORDER XBPDLG_NO_BORDER ;
   HIDE ;
   NOTASKLIST

DCREAD GUI FIT OPTIONS GetOptions PARENT @oDialog EXIT NOAUTORESTORE ;

oDialog:setSize(oStatic:currentSize())
aPos := DC_CalcAbsolutePosition(mp1,oXbp)
aPos[2] -= (oDialog:currentSize()[2] + 20)
oDialog:setPos( aPos )
oDialog:show()

RETURN oDialog


